@@ -1,24 +1,22 @@
 """
 The image module supports basic image loading, rescaling and display
 operations.
-
 """
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
-import six
-from six.moves.urllib.parse import urlparse
-from six.moves.urllib.request import urlopen
 from io import BytesIO
-
 from math import ceil
 import os
+import logging
+import urllib.parse
+import urllib.request
+import warnings
 
 import numpy as np
 
 from matplotlib import rcParams
 import matplotlib.artist as martist
 from matplotlib.artist import allow_rasterization
+from matplotlib.backend_bases import FigureCanvasBase
 import matplotlib.colors as mcolors
 import matplotlib.cm as cm
 import matplotlib.cbook as cbook
@@ -34,6 +32,8 @@ from matplotlib._image import *
 from matplotlib.transforms import (Affine2D, BboxBase, Bbox, BboxTransform,
                                    IdentityTransform, TransformedBbox)
 
+_log = logging.getLogger(__name__)
+
 # map interpolation strings to module constants
 _interpd_ = {
     'none': _image.NEAREST,  # fall back to nearest when not supported
@@ -180,21 +180,6 @@ def _rgb_to_rgba(A):
 class _ImageBase(martist.Artist, cm.ScalarMappable):
     zorder = 0
 
-    @property
-    @cbook.deprecated("2.1")
-    def _interpd(self):
-        return _interpd_
-
-    @property
-    @cbook.deprecated("2.1")
-    def _interpdr(self):
-        return {v: k for k, v in six.iteritems(_interpd_)}
-
-    @property
-    @cbook.deprecated("2.1", alternative="mpl.image.interpolation_names")
-    def iterpnames(self):
-        return interpolations_names
-
     def __str__(self):
         return "AxesImage(%g,%g;%gx%g)" % tuple(self.axes.bbox.bounds)
 
@@ -203,7 +188,7 @@ class _ImageBase(martist.Artist, cm.ScalarMappable):
                  norm=None,
                  interpolation=None,
                  origin=None,
-                 filternorm=1,
+                 filternorm=True,
                  filterrad=4.0,
                  resample=False,
                  **kwargs
@@ -238,7 +223,7 @@ class _ImageBase(martist.Artist, cm.ScalarMappable):
         self.update(kwargs)
 
     def __getstate__(self):
-        state = super(_ImageBase, self).__getstate__()
+        state = super().__getstate__()
         # We can't pickle the C Image cached object.
         state['_imcache'] = None
         return state
@@ -252,10 +237,11 @@ class _ImageBase(martist.Artist, cm.ScalarMappable):
 
     def set_alpha(self, alpha):
         """
-        Set the alpha value used for blending - not supported on
-        all backends
+        Set the alpha value used for blending - not supported on all backends.
 
-        ACCEPTS: float
+        Parameters
+        ----------
+        alpha : float
         """
         martist.Artist.set_alpha(self, alpha)
         self._imcache = None
@@ -278,8 +264,8 @@ class _ImageBase(martist.Artist, cm.ScalarMappable):
         and magnified by the magnification factor.
 
         `A` may be a greyscale image (MxN) with a dtype of `float32`,
-        `float64`, `uint16` or `uint8`, or an RGBA image (MxNx4) with
-        a dtype of `float32`, `float64`, or `uint8`.
+        `float64`, `float128`, `uint16` or `uint8`, or an RGBA image (MxNx4)
+        with a dtype of `float32`, `float64`, `float128`, or `uint8`.
 
         If `unsampled` is True, the image will not be scaled, but an
         appropriate affine transformation will be returned instead.
@@ -364,17 +350,32 @@ class _ImageBase(martist.Artist, cm.ScalarMappable):
 
                 # TODO slice input array first
                 inp_dtype = A.dtype
+                a_min = A.min()
+                a_max = A.max()
+                # figure out the type we should scale to.  For floats,
+                # leave as is.  For integers cast to an appropriate-sized
+                # float.  Small integers get smaller floats in an attempt
+                # to keep the memory footprint reasonable.
+                if a_min is np.ma.masked:
+                    # all masked, so values don't matter
+                    a_min, a_max = np.int32(0), np.int32(1)
                 if inp_dtype.kind == 'f':
                     scaled_dtype = A.dtype
+                    # Cast to float64
+                    if A.dtype not in (np.float32, np.float16):
+                        if A.dtype != np.float64:
+                            warnings.warn(
+                                "Casting input data from '{0}' to 'float64'"
+                                "for imshow".format(A.dtype))
+                        scaled_dtype = np.float64
                 else:
-                    scaled_dtype = np.float32
-
-                a_min = A.min()
-                if a_min is np.ma.masked:
-                    a_min, a_max = 0, 1  # all masked, so values don't matter
-                else:
-                    a_min = a_min.astype(scaled_dtype)
-                    a_max = A.max().astype(scaled_dtype)
+                    # probably an integer of some type.
+                    da = a_max.astype(np.float64) - a_min.astype(np.float64)
+                    if da > 1e8:
+                        # give more breathing room if a big dynamic range
+                        scaled_dtype = np.float64
+                    else:
+                        scaled_dtype = np.float32
 
                 # scale the input data to [.1, .9].  The Agg
                 # interpolators clip to [0, 1] internally, use a
@@ -385,7 +386,35 @@ class _ImageBase(martist.Artist, cm.ScalarMappable):
                 # scaled data
                 A_scaled = np.empty(A.shape, dtype=scaled_dtype)
                 A_scaled[:] = A
+                # clip scaled data around norm if necessary.
+                # This is necessary for big numbers at the edge of
+                # float64's ability to represent changes.  Applying
+                # a norm first would be good, but ruins the interpolation
+                # of over numbers.
+                self.norm.autoscale_None(A)
+                dv = (np.float64(self.norm.vmax) -
+                      np.float64(self.norm.vmin))
+                vmid = self.norm.vmin + dv / 2
+                fact = 1e7 if scaled_dtype == np.float64 else 1e4
+                newmin = vmid - dv * fact
+                if newmin < a_min:
+                    newmin = None
+                else:
+                    a_min = np.float64(newmin)
+                newmax = vmid + dv * fact
+                if newmax > a_max:
+                    newmax = None
+                else:
+                    a_max = np.float64(newmax)
+                if newmax is not None or newmin is not None:
+                    A_scaled = np.clip(A_scaled, newmin, newmax)
+
                 A_scaled -= a_min
+                # a_min and a_max might be ndarray subclasses so use
+                # asscalar to avoid errors
+                a_min = np.asscalar(a_min.astype(scaled_dtype))
+                a_max = np.asscalar(a_max.astype(scaled_dtype))
+
                 if a_min != a_max:
                     A_scaled /= ((a_max - a_min) / 0.8)
                 A_scaled += 0.1
@@ -396,8 +425,8 @@ class _ImageBase(martist.Artist, cm.ScalarMappable):
                                 t,
                                 _interpd_[self.get_interpolation()],
                                 self.get_resample(), 1.0,
-                                self.get_filternorm() or 0.0,
-                                self.get_filterrad() or 0.0)
+                                self.get_filternorm(),
+                                self.get_filterrad())
 
                 # we are done with A_scaled now, remove from namespace
                 # to be sure!
@@ -431,8 +460,8 @@ class _ImageBase(martist.Artist, cm.ScalarMappable):
                                 t,
                                 _interpd_[self.get_interpolation()],
                                 True, 1,
-                                self.get_filternorm() or 0.0,
-                                self.get_filterrad() or 0.0)
+                                self.get_filternorm(),
+                                self.get_filterrad())
                 # we are done with the mask, delete from namespace to be sure!
                 del mask
                 # Agg updates the out_mask in place.  If the pixel has
@@ -465,7 +494,7 @@ class _ImageBase(martist.Artist, cm.ScalarMappable):
                 _image.resample(
                     A, output, t, _interpd_[self.get_interpolation()],
                     self.get_resample(), alpha,
-                    self.get_filternorm() or 0.0, self.get_filterrad() or 0.0)
+                    self.get_filternorm(), self.get_filterrad())
 
             # at this point output is either a 2D array of normed data
             # (of int or float)
@@ -592,13 +621,19 @@ class _ImageBase(martist.Artist, cm.ScalarMappable):
         """
         Set the image array.
 
-        ACCEPTS: numpy/PIL Image A
-
         Note that this function does *not* update the normalization used.
+
+        Parameters
+        ----------
+        A : array-like
         """
         # check if data is PIL Image without importing Image
         if hasattr(A, 'getpixel'):
-            self._A = pil_to_array(A)
+            if A.mode == 'L':
+                # greyscale image, but our logic assumes rgba:
+                self._A = pil_to_array(A.convert('RGBA'))
+            else:
+                self._A = pil_to_array(A)
         else:
             self._A = cbook.safe_masked_invalid(A, copy=True)
 
@@ -610,19 +645,37 @@ class _ImageBase(martist.Artist, cm.ScalarMappable):
                 or self._A.ndim == 3 and self._A.shape[-1] in [3, 4]):
             raise TypeError("Invalid dimensions for image data")
 
+        if self._A.ndim == 3:
+            # If the input data has values outside the valid range (after
+            # normalisation), we issue a warning and then clip X to the bounds
+            # - otherwise casting wraps extreme values, hiding outliers and
+            # making reliable interpretation impossible.
+            high = 255 if np.issubdtype(self._A.dtype, np.integer) else 1
+            if self._A.min() < 0 or high < self._A.max():
+                _log.warning(
+                    'Clipping input data to the valid range for imshow with '
+                    'RGB data ([0..1] for floats or [0..255] for integers).'
+                )
+                self._A = np.clip(self._A, 0, high)
+            # Cast unsupported integer types to uint8
+            if self._A.dtype != np.uint8 and np.issubdtype(self._A.dtype,
+                                                           np.integer):
+                self._A = self._A.astype(np.uint8)
+
         self._imcache = None
         self._rgbacache = None
         self.stale = True
 
     def set_array(self, A):
         """
-        Retained for backwards compatibility - use set_data instead
+        Retained for backwards compatibility - use set_data instead.
 
-        ACCEPTS: numpy array A or PIL Image
+        Parameters
+        ----------
+        A : array-like
         """
         # This also needs to be here to override the inherited
-        # cm.ScalarMappable.set_array method so it is not invoked
-        # by mistake.
+        # cm.ScalarMappable.set_array method so it is not invoked by mistake.
 
         self.set_data(A)
 
@@ -646,10 +699,11 @@ class _ImageBase(martist.Artist, cm.ScalarMappable):
         agg, ps and pdf backends and will fall back to 'nearest' mode
         for other backends.
 
-        ACCEPTS: ['nearest' | 'bilinear' | 'bicubic' | 'spline16' |
-          'spline36' | 'hanning' | 'hamming' | 'hermite' | 'kaiser' |
-          'quadric' | 'catrom' | 'gaussian' | 'bessel' | 'mitchell' |
-          'sinc' | 'lanczos' | 'none' |]
+        Parameters
+        ----------
+        s : {'nearest', 'bilinear', 'bicubic', 'spline16', 'spline36', \
+'hanning', 'hamming', 'hermite', 'kaiser', 'quadric', 'catrom', 'gaussian', \
+'bessel', 'mitchell', 'sinc', 'lanczos', 'none'}
 
         """
         if s is None:
@@ -674,7 +728,9 @@ class _ImageBase(martist.Artist, cm.ScalarMappable):
         """
         Set whether or not image resampling is used.
 
-        ACCEPTS: True|False
+        Parameters
+        ----------
+        v : bool
         """
         if v is None:
             v = rcParams['image.resample']
@@ -687,20 +743,19 @@ class _ImageBase(martist.Artist, cm.ScalarMappable):
 
     def set_filternorm(self, filternorm):
         """
-        Set whether the resize filter norms the weights -- see
-        help for imshow
+        Set whether the resize filter normalizes the weights.
 
-        ACCEPTS: 0 or 1
+        See help for `~.Axes.imshow`.
+
+        Parameters
+        ----------
+        filternorm : bool
         """
-        if filternorm:
-            self._filternorm = 1
-        else:
-            self._filternorm = 0
-
+        self._filternorm = bool(filternorm)
         self.stale = True
 
     def get_filternorm(self):
-        """Return the filternorm setting."""
+        """Return whether the resize filter normalizes the weights."""
         return self._filternorm
 
     def set_filterrad(self, filterrad):
@@ -708,7 +763,9 @@ class _ImageBase(martist.Artist, cm.ScalarMappable):
         Set the resize filter radius only applicable to some
         interpolation schemes -- see help for imshow
 
-        ACCEPTS: positive float
+        Parameters
+        ----------
+        filterrad : positive float
         """
         r = float(filterrad)
         if r <= 0:
@@ -753,7 +810,7 @@ class AxesImage(_ImageBase):
 
         self._extent = extent
 
-        super(AxesImage, self).__init__(
+        super().__init__(
             ax,
             cmap=cmap,
             norm=norm,
@@ -842,15 +899,14 @@ class AxesImage(_ImageBase):
 
 
 class NonUniformImage(AxesImage):
-    def __init__(self, ax, **kwargs):
+    def __init__(self, ax, *, interpolation='nearest', **kwargs):
         """
         kwargs are identical to those for AxesImage, except
         that 'nearest' and 'bilinear' are the only supported 'interpolation'
         options.
         """
-        interp = kwargs.pop('interpolation', 'nearest')
-        super(NonUniformImage, self).__init__(ax, **kwargs)
-        self.set_interpolation(interp)
+        super().__init__(ax, **kwargs)
+        self.set_interpolation(interpolation)
 
     def _check_unsampled_image(self, renderer):
         """
@@ -878,7 +934,7 @@ class NonUniformImage(AxesImage):
             if A.dtype != np.uint8:
                 A = (255*A).astype(np.uint8)
             if A.shape[2] == 3:
-                B = np.zeros(tuple(list(A.shape[0:2]) + [4]), np.uint8)
+                B = np.zeros(tuple([*A.shape[0:2], 4]), np.uint8)
                 B[:, :, 0:3] = A
                 B[:, :, 3] = 255
                 A = B
@@ -931,6 +987,12 @@ class NonUniformImage(AxesImage):
         raise NotImplementedError('Method not supported')
 
     def set_interpolation(self, s):
+        """
+        Parameters
+        ----------
+        s : str, None
+            Either 'nearest', 'bilinear', or ``None``.
+        """
         if s is not None and s not in ('nearest', 'bilinear'):
             raise NotImplementedError('Only nearest neighbor and '
                                       'bilinear interpolations are supported')
@@ -950,12 +1012,12 @@ class NonUniformImage(AxesImage):
     def set_norm(self, norm):
         if self._A is not None:
             raise RuntimeError('Cannot change colors after loading data')
-        super(NonUniformImage, self).set_norm(norm)
+        super().set_norm(norm)
 
     def set_cmap(self, cmap):
         if self._A is not None:
             raise RuntimeError('Cannot change colors after loading data')
-        super(NonUniformImage, self).set_cmap(cmap)
+        super().set_cmap(cmap)
 
 
 class PcolorImage(AxesImage):
@@ -981,7 +1043,7 @@ class PcolorImage(AxesImage):
 
         Additional kwargs are matplotlib.artist properties
         """
-        super(PcolorImage, self).__init__(ax, norm=norm, cmap=cmap)
+        super().__init__(ax, norm=norm, cmap=cmap)
         self.update(kwargs)
         if A is not None:
             self.set_data(x, y, A)
@@ -1109,7 +1171,7 @@ class FigureImage(_ImageBase):
 
         kwargs are an optional list of Artist keyword args
         """
-        super(FigureImage, self).__init__(
+        super().__init__(
             None,
             norm=norm,
             cmap=cmap,
@@ -1179,7 +1241,7 @@ class BboxImage(_ImageBase):
         kwargs are an optional list of Artist keyword args
 
         """
-        super(BboxImage, self).__init__(
+        super().__init__(
             None,
             cmap=cmap,
             norm=norm,
@@ -1240,39 +1302,39 @@ def imread(fname, format=None):
     """
     Read an image from a file into an array.
 
-    *fname* may be a string path, a valid URL, or a Python
-    file-like object.  If using a file object, it must be opened in binary
-    mode.
+    Parameters
+    ----------
+    fname : str or file-like
+        The image file to read. This can be a filename, a URL or a Python
+        file-like object opened in read-binary mode.
+    format : str, optional
+        The image file format assumed for reading the data. If not
+        given, the format is deduced from the filename.  If nothing can
+        be deduced, PNG is tried.
 
-    If *format* is provided, will try to read file of that type,
-    otherwise the format is deduced from the filename.  If nothing can
-    be deduced, PNG is tried.
+    Returns
+    -------
+    imagedata : :class:`numpy.array`
+        The image data. The returned array has shape
 
-    Return value is a :class:`numpy.array`.  For grayscale images, the
-    return array is MxN.  For RGB images, the return value is MxNx3.
-    For RGBA images the return value is MxNx4.
+        - (M, N) for grayscale images.
+        - (M, N, 3) for RGB images.
+        - (M, N, 4) for RGBA images.
 
-    matplotlib can only read PNGs natively, but if `PIL
-    <http://www.pythonware.com/products/pil/>`_ is installed, it will
-    use it to load the image and return an array (if possible) which
-    can be used with :func:`~matplotlib.pyplot.imshow`. Note, URL strings
-    may not be compatible with PIL. Check the PIL documentation for more
-    information.
+    Notes
+    -----
+    Matplotlib can only read PNGs natively. Further image formats are
+    supported via the optional dependency on Pillow. Note, URL strings
+    are not compatible with Pillow. Check the `Pillow documentation`_
+    for more information.
+
+    .. _Pillow documentation: http://pillow.readthedocs.io/en/latest/
     """
 
-    def pilread(fname):
-        """try to load the image with PIL or return None"""
-        try:
-            from PIL import Image
-        except ImportError:
-            return None
-        with Image.open(fname) as image:
-            return pil_to_array(image)
-
     handlers = {'png': _png.read_png, }
     if format is None:
-        if isinstance(fname, six.string_types):
-            parsed = urlparse(fname)
+        if isinstance(fname, str):
+            parsed = urllib.parse.urlparse(fname)
             # If the string is a URL, assume png
             if len(parsed.scheme) > 1:
                 ext = 'png'
@@ -1287,24 +1349,26 @@ def imread(fname, format=None):
     else:
         ext = format
 
-    if ext not in handlers:
-        im = pilread(fname)
-        if im is None:
+    if ext not in handlers:  # Try to load the image with PIL.
+        try:
+            from PIL import Image
+        except ImportError:
             raise ValueError('Only know how to handle extensions: %s; '
                              'with Pillow installed matplotlib can handle '
                              'more images' % list(handlers))
-        return im
+        with Image.open(fname) as image:
+            return pil_to_array(image)
 
     handler = handlers[ext]
 
     # To handle Unicode filenames, we pass a file object to the PNG
     # reader extension, since Python handles them quite well, but it's
     # tricky in C.
-    if isinstance(fname, six.string_types):
-        parsed = urlparse(fname)
+    if isinstance(fname, str):
+        parsed = urllib.parse.urlparse(fname)
         # If fname is a URL, download the data
         if len(parsed.scheme) > 1:
-            fd = BytesIO(urlopen(fname).read())
+            fd = BytesIO(urllib.request.urlopen(fname).read())
             return handler(fd)
         else:
             with open(fname, 'rb') as fd:
@@ -1323,37 +1387,41 @@ def imsave(fname, arr, vmin=None, vmax=None, cmap=None, format=None,
     Parameters
     ----------
     fname : str or file-like
-        Path string to a filename, or a Python file-like object.
-        If *format* is *None* and *fname* is a string, the output
-        format is deduced from the extension of the filename.
+        The filename or a Python file-like object to store the image in.
+        The necessary output format is inferred from the filename extension
+        but may be explicitly overwritten using *format*.
     arr : array-like
-        An MxN (luminance), MxNx3 (RGB) or MxNx4 (RGBA) array.
-    vmin, vmax: [ None | scalar ]
+        The image data. The shape can be one of
+        MxN (luminance), MxNx3 (RGB) or MxNx4 (RGBA).
+    vmin, vmax : scalar, optional
         *vmin* and *vmax* set the color scaling for the image by fixing the
         values that map to the colormap color limits. If either *vmin*
         or *vmax* is None, that limit is determined from the *arr*
         min/max value.
-    cmap : matplotlib.colors.Colormap, optional
-        For example, ``cm.viridis``.  If ``None``, defaults to the
-        ``image.cmap`` rcParam.
-    format : str
-        One of the file extensions supported by the active backend.  Most
-        backends support png, pdf, ps, eps and svg.
-    origin : [ 'upper' | 'lower' ]
-        Indicates whether the ``(0, 0)`` index of the array is in the
-        upper left or lower left corner of the axes.  Defaults to the
-        ``image.origin`` rcParam.
+    cmap : str or `~matplotlib.colors.Colormap`, optional
+        A Colormap instance or registered colormap name. The colormap
+        maps scalar data to colors. It is ignored for RGB(A) data.
+        Defaults to :rc:`image.cmap` ('viridis').
+    format : str, optional
+        The file format, e.g. 'png', 'pdf', 'svg', ... . If not given, the
+        format is deduced form the filename extension in *fname*.
+        See `.Figure.savefig` for details.
+    origin : {'upper', 'lower'}, optional
+        Indicates whether the ``(0, 0)`` index of the array is in the upper
+        left or lower left corner of the axes.  Defaults to :rc:`image.origin`
+        ('upper').
     dpi : int
         The DPI to store in the metadata of the file.  This does not affect the
         resolution of the output image.
     """
     from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
     from matplotlib.figure import Figure
-
-    # Fast path for saving to PNG
-    if (format == 'png' or format is None or
-            isinstance(fname, six.string_types) and
-            fname.lower().endswith('.png')):
+    if isinstance(fname, getattr(os, "PathLike", ())):
+        fname = os.fspath(fname)
+    if (format == 'png'
+        or (format is None
+            and isinstance(fname, str)
+            and fname.lower().endswith('.png'))):
         image = AxesImage(None, cmap=cmap, origin=origin)
         image.set_data(arr)
         image.set_clim(vmin, vmax)
@@ -1367,11 +1435,20 @@ def imsave(fname, arr, vmin=None, vmax=None, cmap=None, format=None,
 
 
 def pil_to_array(pilImage):
-    """Load a PIL image and return it as a numpy array.
+    """Load a `PIL image`_ and return it as a numpy array.
+
+    .. _PIL image: https://pillow.readthedocs.io/en/latest/reference/Image.html
+
+    Returns
+    -------
+    numpy.array
+
+        The array shape depends on the image type:
+
+        - (M, N) for grayscale images.
+        - (M, N, 3) for RGB images.
+        - (M, N, 4) for RGBA images.
 
-    Grayscale images are returned as ``(M, N)`` arrays.  RGB images are
-    returned as ``(M, N, 3)`` arrays.  RGBA images are returned as ``(M, N,
-    4)`` arrays.
     """
     if pilImage.mode in ['RGBA', 'RGBX', 'RGB', 'L']:
         # return MxNx4 RGBA, MxNx3 RBA, or MxN luminance array
@@ -1395,81 +1472,59 @@ def pil_to_array(pilImage):
 def thumbnail(infile, thumbfile, scale=0.1, interpolation='bilinear',
               preview=False):
     """
-    make a thumbnail of image in *infile* with output filename
-    *thumbfile*.
+    Make a thumbnail of image in *infile* with output filename *thumbfile*.
 
-      *infile* the image file -- must be PNG or Pillow-readable if you
-         have `Pillow <http://python-pillow.org/>`_ installed
+    See :doc:`/gallery/misc/image_thumbnail_sgskip`.
 
-      *thumbfile*
-        the thumbnail filename
+    Parameters
+    ----------
+    infile : str or file-like
+        The image file -- must be PNG, Pillow-readable if you have `Pillow
+        <http://python-pillow.org/>`_ installed.
 
-      *scale*
-        the scale factor for the thumbnail
+    thumbfile : str or file-like
+        The thumbnail filename.
 
-      *interpolation*
-        the interpolation scheme used in the resampling
+    scale : float, optional
+        The scale factor for the thumbnail.
 
+    interpolation : str, optional
+        The interpolation scheme used in the resampling. See the
+        *interpolation* parameter of `~.Axes.imshow` for possible values.
 
-      *preview*
-        if True, the default backend (presumably a user interface
-        backend) will be used which will cause a figure to be raised
-        if :func:`~matplotlib.pyplot.show` is called.  If it is False,
-        a pure image backend will be used depending on the extension,
-        'png'->FigureCanvasAgg, 'pdf'->FigureCanvasPdf,
-        'svg'->FigureCanvasSVG
-
-
-    See examples/misc/image_thumbnail.py.
-
-    .. htmlonly::
-
-        :ref:`sphx_glr_gallery_misc_image_thumbnail_sgskip.py`
-
-    Return value is the figure instance containing the thumbnail
+    preview : bool, optional
+        If True, the default backend (presumably a user interface
+        backend) will be used which will cause a figure to be raised if
+        `~matplotlib.pyplot.show` is called.  If it is False, the figure is
+        created using `FigureCanvasBase` and the drawing backend is selected
+        as `~matplotlib.figure.savefig` would normally do.
 
+    Returns
+    -------
+    figure : `~.figure.Figure`
+        The figure instance containing the thumbnail.
     """
-    basedir, basename = os.path.split(infile)
-    baseout, extout = os.path.splitext(thumbfile)
 
     im = imread(infile)
     rows, cols, depth = im.shape
 
-    # this doesn't really matter, it will cancel in the end, but we
-    # need it for the mpl API
+    # This doesn't really matter (it cancels in the end) but the API needs it.
     dpi = 100
 
-    height = float(rows)/dpi*scale
-    width = float(cols)/dpi*scale
-
-    extension = extout.lower()
+    height = rows / dpi * scale
+    width = cols / dpi * scale
 
     if preview:
-        # let the UI backend do everything
+        # Let the UI backend do everything.
         import matplotlib.pyplot as plt
         fig = plt.figure(figsize=(width, height), dpi=dpi)
     else:
-        if extension == '.png':
-            from matplotlib.backends.backend_agg \
-                import FigureCanvasAgg as FigureCanvas
-        elif extension == '.pdf':
-            from matplotlib.backends.backend_pdf \
-                import FigureCanvasPdf as FigureCanvas
-        elif extension == '.svg':
-            from matplotlib.backends.backend_svg \
-                import FigureCanvasSVG as FigureCanvas
-        else:
-            raise ValueError("Can only handle "
-                             "extensions 'png', 'svg' or 'pdf'")
-
         from matplotlib.figure import Figure
         fig = Figure(figsize=(width, height), dpi=dpi)
-        FigureCanvas(fig)
+        FigureCanvasBase(fig)
 
     ax = fig.add_axes([0, 0, 1, 1], aspect='auto',
                       frameon=False, xticks=[], yticks=[])
-
-    basename, ext = os.path.splitext(basename)
     ax.imshow(im, aspect='auto', resample=True, interpolation=interpolation)
     fig.savefig(thumbfile, dpi=dpi)
     return fig
