@@ -1,17 +1,14 @@
 """
 Interpolation inside triangular grids.
 """
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
-import six
-from six.moves import xrange
+import warnings
+
+import numpy as np
 
 from matplotlib.tri import Triangulation
 from matplotlib.tri.trifinder import TriFinder
 from matplotlib.tri.tritools import TriAnalyzer
-import numpy as np
-import warnings
 
 __all__ = ('TriInterpolator', 'LinearTriInterpolator', 'CubicTriInterpolator')
 
@@ -57,7 +54,7 @@ class TriInterpolator(object):
         self._unit_y = 1.0
 
         # Default triangle renumbering: None (= no renumbering)
-        # Renumbering may be used to avoid unecessary computations
+        # Renumbering may be used to avoid unnecessary computations
         # if complex calculations are done inside the Interpolator.
         # Please refer to :meth:`_interpolate_multikeys` for details.
         self._tri_renum = None
@@ -66,7 +63,7 @@ class TriInterpolator(object):
     # (except, if needed, relevant additions).
     # However these methods are only implemented in subclasses to avoid
     # confusion in the documentation.
-    docstring__call__ = """
+    _docstring__call__ = """
         Returns a masked array containing interpolated values at the specified
         x,y points.
 
@@ -79,13 +76,12 @@ class TriInterpolator(object):
         Returns
         -------
         z : np.ma.array
-            Masked array of the same shape as *x* and *y* ; values
-            corresponding to (*x*, *y*) points outside of the triangulation
-            are masked out.
+            Masked array of the same shape as *x* and *y*; values corresponding
+            to (*x*, *y*) points outside of the triangulation are masked out.
 
         """
 
-    docstringgradient = """
+    _docstringgradient = r"""
         Returns a list of 2 masked arrays containing interpolated derivatives
         at the specified x,y points.
 
@@ -98,12 +94,12 @@ class TriInterpolator(object):
         Returns
         -------
         dzdx, dzdy : np.ma.array
-            2  masked arrays of the same shape as *x* and *y* ; values
+            2 masked arrays of the same shape as *x* and *y*; values
             corresponding to (x,y) points outside of the triangulation
             are masked out.
             The first returned array contains the values of
-            :math:`\\frac{\\partial z}{\\partial x}` and the second those of
-            :math:`\\frac{\\partial z}{\\partial y}`.
+            :math:`\frac{\partial z}{\partial x}` and the second those of
+            :math:`\frac{\partial z}{\partial y}`.
 
         """
 
@@ -166,7 +162,7 @@ class TriInterpolator(object):
         x = np.asarray(x, dtype=np.float64)
         y = np.asarray(y, dtype=np.float64)
         sh_ret = x.shape
-        if (x.shape != y.shape):
+        if x.shape != y.shape:
             raise ValueError("x and y shall have same shapes."
                              " Given: {0} and {1}".format(x.shape, y.shape))
         x = np.ravel(x)
@@ -179,7 +175,7 @@ class TriInterpolator(object):
         if tri_index is None:
             tri_index = self._trifinder(x, y)
         else:
-            if (tri_index.shape != sh_ret):
+            if tri_index.shape != sh_ret:
                 raise ValueError(
                     "tri_index array is provided and shall"
                     " have same shape as x and y. Given: "
@@ -274,12 +270,12 @@ class LinearTriInterpolator(TriInterpolator):
     def __call__(self, x, y):
         return self._interpolate_multikeys(x, y, tri_index=None,
                                            return_keys=('z',))[0]
-    __call__.__doc__ = TriInterpolator.docstring__call__
+    __call__.__doc__ = TriInterpolator._docstring__call__
 
     def gradient(self, x, y):
         return self._interpolate_multikeys(x, y, tri_index=None,
                                            return_keys=('dzdx', 'dzdy'))
-    gradient.__doc__ = TriInterpolator.docstringgradient
+    gradient.__doc__ = TriInterpolator._docstringgradient
 
     def _interpolate_single_key(self, return_key, tri_index, x, y):
         if return_key == 'z':
@@ -295,7 +291,7 @@ class LinearTriInterpolator(TriInterpolator):
 
 
 class CubicTriInterpolator(TriInterpolator):
-    """
+    r"""
     A CubicTriInterpolator performs cubic interpolation on triangular grids.
 
     In one-dimension - on a segment - a cubic interpolating function is
@@ -370,11 +366,11 @@ class CubicTriInterpolator(TriInterpolator):
 
         .. math::
 
-            E(z) = \\ \\frac{1}{2} \\int_{\\Omega}   \\left(
-            \\left( \\frac{\\partial^2{z}}{\\partial{x}^2} \\right)^2 +
-            \\left( \\frac{\\partial^2{z}}{\\partial{y}^2} \\right)^2 +
-            2\\left( \\frac{\\partial^2{z}}{\\partial{y}\\partial{x}}
-            \\right)^2 \\right)  dx\\,dy
+            E(z) = \frac{1}{2} \int_{\Omega} \left(
+                \left( \frac{\partial^2{z}}{\partial{x}^2} \right)^2 +
+                \left( \frac{\partial^2{z}}{\partial{y}^2} \right)^2 +
+                2\left( \frac{\partial^2{z}}{\partial{y}\partial{x}} \right)^2
+            \right) dx\,dy
 
     If the case *kind* ='geom' is chosen by the user, a simple geometric
     approximation is used (weighted average of the triangle normal
@@ -417,10 +413,10 @@ class CubicTriInterpolator(TriInterpolator):
         self._z = self._z[~node_mask]
 
         # Computing scale factors
-        self._unit_x = np.max(compressed_x) - np.min(compressed_x)
-        self._unit_y = np.max(compressed_y) - np.min(compressed_y)
-        self._pts = np.vstack((compressed_x/float(self._unit_x),
-                               compressed_y/float(self._unit_y))).T
+        self._unit_x = np.ptp(compressed_x)
+        self._unit_y = np.ptp(compressed_y)
+        self._pts = np.column_stack([compressed_x / self._unit_x,
+                                     compressed_y / self._unit_y])
         # Computing triangle points
         self._tris_pts = self._pts[self._triangles]
         # Computing eccentricities
@@ -433,21 +429,12 @@ class CubicTriInterpolator(TriInterpolator):
     def __call__(self, x, y):
         return self._interpolate_multikeys(x, y, tri_index=None,
                                            return_keys=('z',))[0]
-    __call__.__doc__ = TriInterpolator.docstring__call__
+    __call__.__doc__ = TriInterpolator._docstring__call__
 
     def gradient(self, x, y):
         return self._interpolate_multikeys(x, y, tri_index=None,
                                            return_keys=('dzdx', 'dzdy'))
-    gradient.__doc__ = TriInterpolator.docstringgradient + """
-
-        Examples
-        --------
-        An example of effective application is shown below (plot of the
-        direction of the vector field derivated from a known potential field):
-
-        .. plot:: mpl_examples/pylab_examples/trigradient_demo.py
-
-        """
+    gradient.__doc__ = TriInterpolator._docstringgradient
 
     def _interpolate_single_key(self, return_key, tri_index, x, y):
         tris_pts = self._tris_pts[tri_index]
@@ -478,7 +465,7 @@ class CubicTriInterpolator(TriInterpolator):
             Choice of the _DOF_estimator subclass to perform the gradient
             estimation.
         dz: tuple of array_likes (dzdx, dzdy), optional
-            Used only if *kind=user ; in this case passed to the
+            Used only if *kind*=user; in this case passed to the
             :class:`_DOF_estimator_user`.
 
         Returns
@@ -498,7 +485,7 @@ class CubicTriInterpolator(TriInterpolator):
         elif kind == 'min_E':
             TE = _DOF_estimator_min_E(self)
         else:
-            raise ValueError("CubicTriInterpolator *kind* proposed: {0} ; "
+            raise ValueError("CubicTriInterpolator *kind* proposed: {0}; "
                              "should be one of: "
                              "'user', 'geom', 'min_E'".format(kind))
         return TE.compute_dof_from_df()
@@ -526,12 +513,9 @@ class CubicTriInterpolator(TriInterpolator):
 
         a = tris_pts[:, 1, :] - tris_pts[:, 0, :]
         b = tris_pts[:, 2, :] - tris_pts[:, 0, :]
-        abT = np.concatenate([np.expand_dims(a, ndim+1),
-                              np.expand_dims(b, ndim+1)], ndim+1)
+        abT = np.stack([a, b], axis=-1)
         ab = _transpose_vectorized(abT)
-        x = np.expand_dims(x, ndim)
-        y = np.expand_dims(y, ndim)
-        OM = np.concatenate([x, y], ndim) - tris_pts[:, 0, :]
+        OM = np.stack([x, y], axis=1) - tris_pts[:, 0, :]
 
         metric = _prod_vectorized(ab, abT)
         # Here we try to deal with the colinear cases.
@@ -793,7 +777,7 @@ class _ReducedHCT_Element():
         dsdksi = _roll_vectorized(prod, 3*subtri, axis=0)
         dfdksi = _prod_vectorized(dofs, dsdksi)
         # In global coordinates:
-        # Here we try to deal with the simpliest colinear cases, returning a
+        # Here we try to deal with the simplest colinear cases, returning a
         # null matrix.
         J_inv = _safe_inv22_vectorized(J)
         dfdx = _prod_vectorized(J_inv, _transpose_vectorized(dfdksi))
@@ -907,7 +891,7 @@ class _ReducedHCT_Element():
         pts = self.gauss_pts
         for igauss in range(self.n_gauss):
             alpha = np.tile(pts[igauss, :], n).reshape(n, 3)
-            alpha = np.expand_dims(alpha, 3)
+            alpha = np.expand_dims(alpha, 2)
             weight = weights[igauss]
             d2Skdksi2 = self.get_d2Sidksij2(alpha, ecc)
             d2Skdx2 = _prod_vectorized(d2Skdksi2, H_rot)
@@ -934,7 +918,7 @@ class _ReducedHCT_Element():
         to global coordinates.
         if *return_area* is True, returns also the triangle area (0.5*det(J))
         """
-        # Here we try to deal with the simpliest colinear cases ; a null
+        # Here we try to deal with the simplest colinear cases; a null
         # energy and area is imposed.
         J_inv = _safe_inv22_vectorized(J)
         Ji00 = J_inv[:, 0, 0]
@@ -1153,9 +1137,9 @@ class _DOF_estimator_geom(_DOF_estimator):
             # In the below formula we could take modulo 2. but
             # modulo 1. is safer regarding round-off errors (flat triangles).
             angle = np.abs(np.mod((alpha2-alpha1) / np.pi, 1.))
-            # Weight proportional to angle up np.pi/2 ; null weight for
-            # degenerated cases 0. and np.pi (Note that `angle` is normalized
-            # by np.pi)
+            # Weight proportional to angle up np.pi/2; null weight for
+            # degenerated cases 0 and np.pi (note that `angle` is normalized
+            # by np.pi).
             weights[:, ipt] = 0.5 - np.abs(angle-0.5)
         return weights
 
@@ -1171,7 +1155,7 @@ class _DOF_estimator_geom(_DOF_estimator):
         dM1 = tris_pts[:, 1, :] - tris_pts[:, 0, :]
         dM2 = tris_pts[:, 2, :] - tris_pts[:, 0, :]
         dM = np.dstack([dM1, dM2])
-        # Here we try to deal with the simpliest colinear cases: a null
+        # Here we try to deal with the simplest colinear cases: a null
         # gradient is assumed in this case.
         dM_inv = _safe_inv22_vectorized(dM)
 
@@ -1216,9 +1200,9 @@ class _DOF_estimator_min_E(_DOF_estimator_geom):
             J, eccs, triangles, Uc)
 
         # Building sparse matrix and solving minimization problem
-        # We could use scipy.sparse direct solver ; however to avoid this
+        # We could use scipy.sparse direct solver; however to avoid this
         # external dependency an implementation of a simple PCG solver with
-        # a simplendiagonal Jocabi preconditioner is implemented.
+        # a simple diagonal Jacobi preconditioner is implemented.
         tol = 1.e-10
         n_dof = Ff.shape[0]
         Kff_coo = _Sparse_Matrix_coo(Kff_vals, Kff_rows, Kff_cols,
@@ -1244,7 +1228,7 @@ class _DOF_estimator_min_E(_DOF_estimator_geom):
 
 # The following private :class:_Sparse_Matrix_coo and :func:_cg provide
 # a PCG sparse solver for (symmetric) elliptic problems.
-class _Sparse_Matrix_coo:
+class _Sparse_Matrix_coo(object):
     def __init__(self, vals, rows, cols, shape):
         """
         Creates a sparse matrix in coo format
@@ -1265,12 +1249,9 @@ class _Sparse_Matrix_coo:
         *V* dense vector of shape (self.m,)
         """
         assert V.shape == (self.m,)
-        # For a more generic implementation we could use below kw argument
-        # minlength=self.m of bincount ; however:
-        # - it is new in numpy 1.6
-        # - it is unecessary when each row have at least 1 entry in global
-        #   matrix, which is the case here.
-        return np.bincount(self.rows, weights=self.vals*V[self.cols])
+        return np.bincount(self.rows,
+                           weights=self.vals*V[self.cols],
+                           minlength=self.m)
 
     def compress_csc(self):
         """
@@ -1334,14 +1315,14 @@ def _cg(A, b, x0=None, tol=1.e-10, maxiter=1000):
         Right hand side of the linear system.
 
     Returns
-    ----------
+    -------
     x: array.
         The converged solution.
     err: float
         The absolute error np.linalg.norm(A.dot(x) - b)
 
     Other parameters
-    ----------
+    ----------------
     x0: array.
         Starting guess for the solution.
     tol: float.
@@ -1394,7 +1375,6 @@ def _cg(A, b, x0=None, tol=1.e-10, maxiter=1000):
 
 
 # The following private functions:
-#     :func:`_inv22_vectorized`
 #     :func:`_safe_inv22_vectorized`
 #     :func:`_pseudo_inv22sym_vectorized`
 #     :func:`_prod_vectorized`
@@ -1405,20 +1385,6 @@ def _cg(A, b, x0=None, tol=1.e-10, maxiter=1000):
 #     :func:`_extract_submatrices`
 # provide fast numpy implementation of some standard operations on arrays of
 # matrices - stored as (:, n_rows, n_cols)-shaped np.arrays.
-def _inv22_vectorized(M):
-    """
-    Inversion of arrays of (2,2) matrices.
-    """
-    assert (M.ndim == 3)
-    assert (M.shape[-2:] == (2, 2))
-    M_inv = np.empty_like(M)
-    delta_inv = np.reciprocal(M[:, 0, 0]*M[:, 1, 1] - M[:, 0, 1]*M[:, 1, 0])
-    M_inv[:, 0, 0] = M[:, 1, 1]*delta_inv
-    M_inv[:, 0, 1] = -M[:, 0, 1]*delta_inv
-    M_inv[:, 1, 0] = -M[:, 1, 0]*delta_inv
-    M_inv[:, 1, 1] = M[:, 0, 0]*delta_inv
-    return M_inv
-
 
 # Development note: Dealing with pathologic 'flat' triangles in the
 # CubicTriInterpolator code and impact on (2,2)-matrix inversion functions
@@ -1468,7 +1434,7 @@ def _safe_inv22_vectorized(M):
     prod1 = M[:, 0, 0]*M[:, 1, 1]
     delta = prod1 - M[:, 0, 1]*M[:, 1, 0]
 
-    # We set delta_inv to 0. in case of a rank deficient matrix ; a
+    # We set delta_inv to 0. in case of a rank deficient matrix; a
     # rank-deficient input matrix *M* will lead to a null matrix in output
     rank2 = (np.abs(delta) > 1e-8*np.abs(prod1))
     if np.all(rank2):
@@ -1488,7 +1454,7 @@ def _safe_inv22_vectorized(M):
 
 def _pseudo_inv22sym_vectorized(M):
     """
-    Inversion of arrays of (2,2) SYMMETRIC matrices ; returns the
+    Inversion of arrays of (2,2) SYMMETRIC matrices; returns the
     (Moore-Penrose) pseudo-inverse for rank-deficient matrices.
 
     In case M is of rank 1, we have M = trace(M) x P where P is the orthogonal
@@ -1545,7 +1511,7 @@ def _prod_vectorized(M1, M2):
     assert sh1[-1] == sh2[-2]
 
     ndim1 = len(sh1)
-    t1_index = list(xrange(ndim1-2)) + [ndim1-1, ndim1-2]
+    t1_index = [*range(ndim1-2), ndim1-1, ndim1-2]
     return np.sum(np.transpose(M1, t1_index)[..., np.newaxis] *
                   M2[..., np.newaxis, :], -3)
 
@@ -1561,9 +1527,7 @@ def _transpose_vectorized(M):
     """
     Transposition of an array of matrices *M*.
     """
-    ndim = M.ndim
-    assert ndim == 3
-    return np.transpose(M, [0, ndim-1, ndim-2])
+    return np.transpose(M, [0, 2, 1])
 
 
 def _roll_vectorized(M, roll_indices, axis):
@@ -1605,7 +1569,7 @@ def _to_matrix_vectorized(M):
         M_res[...,i,j] = M[i][j]
     """
     assert isinstance(M, (tuple, list))
-    assert all([isinstance(item, (tuple, list)) for item in M])
+    assert all(isinstance(item, (tuple, list)) for item in M)
     c_vec = np.asarray([len(item) for item in M])
     assert np.all(c_vec-c_vec[0] == 0)
     r = len(M)
